# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import urllib
from google.appengine.api import urlfetch, taskqueue
from google.appengine.api.labs.taskqueue.taskqueue import Task
from google.appengine.api.taskqueue.taskqueue import Queue
from gaebusiness.business import Command


class UrlFecthCommand(Command):
    def __init__(self, url, params={}, method=urlfetch.GET, headers={}, validate_certificate=True):
        super(UrlFecthCommand, self).__init__()
        self._method = method
        self._headers = headers
        self._validate_certificate = validate_certificate
        self._url = url
        self._params = None
        if params:
            encoded_params = urllib.urlencode(params)
            if method in (urlfetch.POST, urlfetch.PUT, urlfetch.PATCH):
                self._params = encoded_params
            else:
                self._url = "%s?%s" % (url, encoded_params)

    def set_up(self):
        self._rpc = urlfetch.create_rpc()
        urlfetch.make_fetch_call(self._rpc, self._url, self._params, method=self._method,
                                 validate_certificate=self._validate_certificate, headers=self._headers)

    def do_business(self, stop_on_error=False):
        self.result = self._rpc.get_result()


class TaskQueueCommand(Command):
    def __init__(self, queue_name, url, **kwargs):
        '''
        kwargs are the same used on Task class
        (https://developers.google.com/appengine/docs/python/taskqueue/tasks#Task)
        '''
        super(TaskQueueCommand, self).__init__()
        self._task = Task(url=url, **kwargs)
        self._queue_name = queue_name


    def set_up(self):
        self._rpc = taskqueue.create_rpc()
        q=Queue(self._queue_name)
        q.add_async(self._task, rpc=self._rpc)

    def do_business(self, stop_on_error=False):
        self._rpc.get_result()





# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from google.appengine.ext import ndb


def to_model_list(models):
    if models is None:
        return []
    return [models] if isinstance(models, ndb.Model) else models


class Command(object):
    def __init__(self,**kwargs):
        self.errors = {}
        self.result = None
        for k,v in kwargs.iteritems():
            setattr(self,k,v)

    def add_error(self, key, msg):
        self.errors[key] = msg

    def set_up(self):
        '''
        Must set_up data for business.
        It should fetch data asyncrounously if needed
        '''
        pass

    def do_business(self, stop_on_error=False):
        '''
        Must do the main business of use case
        '''
        raise NotImplementedError()

    def commit(self):
        '''
        Must return a Model, or a list of it to be commited on DB
        '''
        return []

    def execute(self, stop_on_error=False):
        self.set_up()
        self.do_business(stop_on_error)
        ndb.put_multi(to_model_list(self.commit()))
        return self.errors


class CommandList(Command):
    def __init__(self, commands,**kwargs):
        super(CommandList, self).__init__(**kwargs)
        self.commands = commands


    def set_up(self):
        for cmd in self.commands:
            cmd.set_up()

    def do_business(self, stop_on_error=False):
        for cmd in self.commands:
            if not cmd.errors:
                cmd.do_business(stop_on_error)
            self.errors.update(cmd.errors)
            if stop_on_error and self.errors:
                return self.errors
        return self.errors

    def commit(self):
        to_commit = []
        for cmd in self.commands:
            to_commit.extend(to_model_list(cmd.commit()))
        return to_commit
